@@ -253,6 +253,7 @@ class Priority(StrEnum):
     Defines the urgency with which a task will run.
     """
 
+    immediate = "immediate"
     urgent = "urgent"
     high = "high"
     normal = "normal"
@@ -435,6 +436,14 @@ class TaskSpec(BaseModel, frozen=False):
     Timeout for jobs in the task.
     """
 
+    @field_validator("synchronized_start_timeout", "timeout", mode="before")
+    @classmethod
+    def ensure_nanoseconds(cls, value: Any) -> int:
+        if isinstance(value, str):
+            return parse_duration(value)
+        else:
+            return value
+
     @classmethod
     def new(
         cls,
@@ -499,15 +508,6 @@ class TaskSpec(BaseModel, frozen=False):
                 else:
                     constraints = Constraints(cluster=[cluster])
 
-        # Allow setting timeouts as a string rather than nanoseconds, and assume a string
-        # without units means seconds.
-        if (
-            synchronized_start_timeout_str := kwargs.pop("synchronized_start_timeout", None)
-        ) is not None:
-            kwargs["synchronized_start_timeout"] = parse_duration(synchronized_start_timeout_str)
-        if (timeout_str := kwargs.pop("timeout", None)) is not None:
-            kwargs["timeout"] = parse_duration(timeout_str)
-
         return TaskSpec(
             name=name,
             image=ImageSource(beaker=beaker_image, docker=docker_image),
